/*
decoder 16 feux Dominique Bultez
Base : DCC Decoder de Minabay
Usage des dcc codes (2 codes par feu):
@dcc  @dcc+1   Led  0    1    2
high  low      /    on   off  off vert   1
low   high     /    off  on   off jaune  2
high  high     /    off  off  on  rouge  3
low   low      /    off  off  off eteint 0
valeur @dcc+2@dcc+1 : 3 (rouge), 2 (jaune), 1 (vert), 0 (eteint)
version V01 du 220215
*/

#include <DCC_Decoder.h>
#define kDCC_INTERRUPT    3    // pin 20 for Mega2560

#define NB_FEUX 16
#define NB_LEDS 3

// plage de codes DCC : 100 a 131

#define DCC_CODE_START 100

// DCC_codes1 = DCC_CODE_START + 2*LED_NUM
// DCC_codes2 = DCC_CODE_START + 2*LED_NUM + 1

int dcc_codes[NB_FEUX] = { 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130 };

// Pins des Leds des Feux {vert, jaune, rouge, allumage = LOW }

int pins[NB_FEUX][NB_LEDS] = {
		            { 11, 10, 9 },
                { 8, 7, 6 },
		            { 5, 4, 25 },
		            { 27, 29, 31 },
		            { 28, 30, 32 },
                { 34, 36, 38 },
                { 40, 42, 44 },
                { 46, 48, 50 },
                { 13, 12, 3 },
                { 2, 14, 15 },
                { 16, 17, 18 },               
                { 19, 22, 23 },
                { 24, 26, 33 },
                { 35, 37, 39 },
                { 41, 43, 45 },
		            { 47, 49, 51 }
};  

int STATE_FEUX[NB_FEUX*2];    // NB_FEUX couples = 4 etats, valeurs HIGH ou LOW
boolean VALID_FEUX[NB_FEUX];  // un feu est a traiter si true par Maj_Feux
int NUM_FEUX;


/**
 * Mise a jour des feux
 */
void Maj_Feux() {
  
  for (int i = 0; i < NB_FEUX; i++)  
  {
    if (VALID_FEUX[i] == true)
    {
      // conversion des 2 etats en State compris entre 0  et 3
      int State = STATE_FEUX[i*2] + STATE_FEUX[i*2+1]*2;
      // positionnement des Leds 
      switch(State) 
      {
        case 0: // rouge
        digitalWrite(pins[i][0] , HIGH);  //extinction Led
        digitalWrite(pins[i][1] , HIGH);  //extinction Led
        digitalWrite(pins[i][2] , LOW);  //extinction Led
        break;
        case 1: // jaune
        digitalWrite(pins[i][0] , HIGH);  //allumage Led
        digitalWrite(pins[i][1] , LOW);  //extinction Led
        digitalWrite(pins[i][2] , HIGH);  //extinction Led
        break;
        case 2: // jaune
        digitalWrite(pins[i][0] , HIGH);  //extinction Led
        digitalWrite(pins[i][1] , LOW);  //allumage Led
        digitalWrite(pins[i][2] , HIGH);  //extinction Led
        break;
        case 3: // vert
        digitalWrite(pins[i][0] , LOW);  //extinction Led
        digitalWrite(pins[i][1] , HIGH);  //extinction Led
        digitalWrite(pins[i][2] , HIGH);  //allumage Led
        break;
      }
      VALID_FEUX[i] = false;
    }
  }
}



/**
 * Handler DCC
 */
void BasicAccDecoderPacket_Handler(int address, boolean activate, byte data)
{
  // Conversion de l'adresse NMRA en adresse decodeur d'accessoire
  address -= 1;
  address *= 4;
  address += 1;
  address += (data & 0x06) >> 1;
    
  int enable = (data & 0x01) ? HIGH : LOW;
    
  if ((address >= DCC_CODE_START) && (address < DCC_CODE_START + NB_FEUX*2))
  {
    VALID_FEUX[(address - DCC_CODE_START)/2] =  true;
    STATE_FEUX[address - DCC_CODE_START] = enable;
    Serial.print("@ ");
    Serial.print(address);
    Serial.print(" ");
    Serial.print(STATE_FEUX[address - DCC_CODE_START]);
    Serial.print(" feu ");
    for (int i = 0; i < NB_FEUX; i++)  
    {
      if (VALID_FEUX[i] == true)
      {
        Serial.print(i);
        Serial.print(" etat ");
        int State = STATE_FEUX[i*2] + STATE_FEUX[i*2+1]*2;
        Serial.println(State);
      }
    }
  }
}



void setup() {
  
  Serial.begin(115200);

  for (int i=0; i<NB_FEUX; i++)
  {
    STATE_FEUX[i*2] = LOW;  // eteint
    STATE_FEUX[i*2+1] = LOW;  // eteint
    VALID_FEUX[i] = true;
    for (int j=0; j<NB_LEDS; j++)
    {
    pinMode(pins[i][j] , OUTPUT); 
    digitalWrite(pins[i][j] , LOW);  //allumage Led
    delay(50);
    digitalWrite(pins[i][j] , HIGH);  //extinction Led
    delay(50);
    }
  }
    
  
  DCC.SetBasicAccessoryDecoderPacketHandler(BasicAccDecoderPacket_Handler, true);
//  ConfigureDecoder();
  DCC.SetupDecoder( 0x00, 0x00, kDCC_INTERRUPT );
  
  Serial.println("DCC 16 Feux en route (V1)");
}



void loop() {
  static unsigned long timer=0;
  
  ////////////////////////////////////////////////////////////////
  // Loop DCC library
  DCC.loop();
   
  
  ////////////////////////////////////////////////////////////////
  // on change l'etat des Leds toutes les 50 millisecondes
  
  if (millis()-timer>50) {
    Maj_Feux();
    timer=millis();
  }
}

